(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('spark-md5')) :
	typeof define === 'function' && define.amd ? define(['exports', 'spark-md5'], factory) :
	(factory((global.ActiveStorage = {}),global.SparkMD5));
}(this, (function (exports,SparkMD5) { 'use strict';

SparkMD5 = SparkMD5 && SparkMD5.hasOwnProperty('default') ? SparkMD5['default'] : SparkMD5;

var asyncGenerator = function () {
  function AwaitValue(value) {
    this.value = value;
  }

  function AsyncGenerator(gen) {
    var front, back;

    function send(key, arg) {
      return new Promise(function (resolve, reject) {
        var request = {
          key: key,
          arg: arg,
          resolve: resolve,
          reject: reject,
          next: null
        };

        if (back) {
          back = back.next = request;
        } else {
          front = back = request;
          resume(key, arg);
        }
      });
    }

    function resume(key, arg) {
      try {
        var result = gen[key](arg);
        var value = result.value;

        if (value instanceof AwaitValue) {
          Promise.resolve(value.value).then(function (arg) {
            resume("next", arg);
          }, function (arg) {
            resume("throw", arg);
          });
        } else {
          settle(result.done ? "return" : "normal", result.value);
        }
      } catch (err) {
        settle("throw", err);
      }
    }

    function settle(type, value) {
      switch (type) {
        case "return":
          front.resolve({
            value: value,
            done: true
          });
          break;

        case "throw":
          front.reject(value);
          break;

        default:
          front.resolve({
            value: value,
            done: false
          });
          break;
      }

      front = front.next;

      if (front) {
        resume(front.key, front.arg);
      } else {
        back = null;
      }
    }

    this._invoke = send;

    if (typeof gen.return !== "function") {
      this.return = undefined;
    }
  }

  if (typeof Symbol === "function" && Symbol.asyncIterator) {
    AsyncGenerator.prototype[Symbol.asyncIterator] = function () {
      return this;
    };
  }

  AsyncGenerator.prototype.next = function (arg) {
    return this._invoke("next", arg);
  };

  AsyncGenerator.prototype.throw = function (arg) {
    return this._invoke("throw", arg);
  };

  AsyncGenerator.prototype.return = function (arg) {
    return this._invoke("return", arg);
  };

  return {
    wrap: function (fn) {
      return function () {
        return new AsyncGenerator(fn.apply(this, arguments));
      };
    },
    await: function (value) {
      return new AwaitValue(value);
    }
  };
}();





var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();

var fileSlice = File.prototype.slice || File.prototype.mozSlice || File.prototype.webkitSlice;

var FileChecksum = function () {
  createClass(FileChecksum, null, [{
    key: "create",
    value: function create(file, callback) {
      var instance = new FileChecksum(file);
      instance.create(callback);
    }
  }]);

  function FileChecksum(file) {
    classCallCheck(this, FileChecksum);

    this.file = file;
    this.chunkSize = 2097152; // 2MB
    this.chunkCount = Math.ceil(this.file.size / this.chunkSize);
    this.chunkIndex = 0;
  }

  createClass(FileChecksum, [{
    key: "create",
    value: function create(callback) {
      var _this = this;

      this.callback = callback;
      this.md5Buffer = new SparkMD5.ArrayBuffer();
      this.fileReader = new FileReader();
      this.fileReader.addEventListener("load", function (event) {
        return _this.fileReaderDidLoad(event);
      });
      this.fileReader.addEventListener("error", function (event) {
        return _this.fileReaderDidError(event);
      });
      this.readNextChunk();
    }
  }, {
    key: "fileReaderDidLoad",
    value: function fileReaderDidLoad(event) {
      this.md5Buffer.append(event.target.result);

      if (!this.readNextChunk()) {
        var binaryDigest = this.md5Buffer.end(true);
        var base64digest = btoa(binaryDigest);
        this.callback(null, base64digest);
      }
    }
  }, {
    key: "fileReaderDidError",
    value: function fileReaderDidError(event) {
      this.callback("Error reading " + this.file.name);
    }
  }, {
    key: "readNextChunk",
    value: function readNextChunk() {
      if (this.chunkIndex < this.chunkCount || this.chunkIndex == 0 && this.chunkCount == 0) {
        var start = this.chunkIndex * this.chunkSize;
        var end = Math.min(start + this.chunkSize, this.file.size);
        var bytes = fileSlice.call(this.file, start, end);
        this.fileReader.readAsArrayBuffer(bytes);
        this.chunkIndex++;
        return true;
      } else {
        return false;
      }
    }
  }]);
  return FileChecksum;
}();

function getMetaValue(name) {
  var element = findElement(document.head, "meta[name=\"" + name + "\"]");
  if (element) {
    return element.getAttribute("content");
  }
}

function findElements(root, selector) {
  if (typeof root == "string") {
    selector = root;
    root = document;
  }
  var elements = root.querySelectorAll(selector);
  return toArray$1(elements);
}

function findElement(root, selector) {
  if (typeof root == "string") {
    selector = root;
    root = document;
  }
  return root.querySelector(selector);
}

function dispatchEvent(element, type) {
  var eventInit = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  var disabled = element.disabled;
  var bubbles = eventInit.bubbles,
      cancelable = eventInit.cancelable,
      detail = eventInit.detail;

  var event = document.createEvent("Event");

  event.initEvent(type, bubbles || true, cancelable || true);
  event.detail = detail || {};

  try {
    element.disabled = false;
    element.dispatchEvent(event);
  } finally {
    element.disabled = disabled;
  }

  return event;
}

function toArray$1(value) {
  if (Array.isArray(value)) {
    return value;
  } else if (Array.from) {
    return Array.from(value);
  } else {
    return [].slice.call(value);
  }
}

var BlobRecord = function () {
  function BlobRecord(file, checksum, url) {
    var _this = this;

    classCallCheck(this, BlobRecord);

    this.file = file;

    this.attributes = {
      filename: file.name,
      content_type: file.type,
      byte_size: file.size,
      checksum: checksum
    };

    this.xhr = new XMLHttpRequest();
    this.xhr.open("POST", url, true);
    this.xhr.responseType = "json";
    this.xhr.setRequestHeader("Content-Type", "application/json");
    this.xhr.setRequestHeader("Accept", "application/json");
    this.xhr.setRequestHeader("X-Requested-With", "XMLHttpRequest");
    this.xhr.setRequestHeader("X-CSRF-Token", getMetaValue("csrf-token"));
    this.xhr.addEventListener("load", function (event) {
      return _this.requestDidLoad(event);
    });
    this.xhr.addEventListener("error", function (event) {
      return _this.requestDidError(event);
    });
  }

  createClass(BlobRecord, [{
    key: "create",
    value: function create(callback) {
      this.callback = callback;
      this.xhr.send(JSON.stringify({ blob: this.attributes }));
    }
  }, {
    key: "requestDidLoad",
    value: function requestDidLoad(event) {
      if (this.status >= 200 && this.status < 300) {
        var response = this.response;
        var direct_upload = response.direct_upload;

        delete response.direct_upload;
        this.attributes = response;
        this.directUploadData = direct_upload;
        this.callback(null, this.toJSON());
      } else {
        this.requestDidError(event);
      }
    }
  }, {
    key: "requestDidError",
    value: function requestDidError(event) {
      this.callback("Error creating Blob for \"" + this.file.name + "\". Status: " + this.status);
    }
  }, {
    key: "toJSON",
    value: function toJSON() {
      var result = {};
      for (var key in this.attributes) {
        result[key] = this.attributes[key];
      }
      return result;
    }
  }, {
    key: "status",
    get: function get$$1() {
      return this.xhr.status;
    }
  }, {
    key: "response",
    get: function get$$1() {
      var _xhr = this.xhr,
          responseType = _xhr.responseType,
          response = _xhr.response;

      if (responseType == "json") {
        return response;
      } else {
        // Shim for IE 11: https://connect.microsoft.com/IE/feedback/details/794808
        return JSON.parse(response);
      }
    }
  }]);
  return BlobRecord;
}();

var BlobUpload = function () {
  function BlobUpload(blob) {
    var _this = this;

    classCallCheck(this, BlobUpload);

    this.blob = blob;
    this.file = blob.file;

    var _blob$directUploadDat = blob.directUploadData,
        url = _blob$directUploadDat.url,
        headers = _blob$directUploadDat.headers;


    this.xhr = new XMLHttpRequest();
    this.xhr.open("PUT", url, true);
    this.xhr.responseType = "text";
    for (var key in headers) {
      this.xhr.setRequestHeader(key, headers[key]);
    }
    this.xhr.addEventListener("load", function (event) {
      return _this.requestDidLoad(event);
    });
    this.xhr.addEventListener("error", function (event) {
      return _this.requestDidError(event);
    });
  }

  createClass(BlobUpload, [{
    key: "create",
    value: function create(callback) {
      this.callback = callback;
      this.xhr.send(this.file.slice());
    }
  }, {
    key: "requestDidLoad",
    value: function requestDidLoad(event) {
      var _xhr = this.xhr,
          status = _xhr.status,
          response = _xhr.response;

      if (status >= 200 && status < 300) {
        this.callback(null, response);
      } else {
        this.requestDidError(event);
      }
    }
  }, {
    key: "requestDidError",
    value: function requestDidError(event) {
      this.callback("Error storing \"" + this.file.name + "\". Status: " + this.xhr.status);
    }
  }]);
  return BlobUpload;
}();

var id = 0;

var DirectUpload = function () {
  function DirectUpload(file, url, delegate) {
    classCallCheck(this, DirectUpload);

    this.id = ++id;
    this.file = file;
    this.url = url;
    this.delegate = delegate;
  }

  createClass(DirectUpload, [{
    key: "create",
    value: function create(callback) {
      var _this = this;

      FileChecksum.create(this.file, function (error, checksum) {
        if (error) {
          callback(error);
          return;
        }

        var blob = new BlobRecord(_this.file, checksum, _this.url);
        notify(_this.delegate, "directUploadWillCreateBlobWithXHR", blob.xhr);

        blob.create(function (error) {
          if (error) {
            callback(error);
          } else {
            var upload = new BlobUpload(blob);
            notify(_this.delegate, "directUploadWillStoreFileWithXHR", upload.xhr);
            upload.create(function (error) {
              if (error) {
                callback(error);
              } else {
                callback(null, blob.toJSON());
              }
            });
          }
        });
      });
    }
  }]);
  return DirectUpload;
}();

function notify(object, methodName) {
  if (object && typeof object[methodName] == "function") {
    for (var _len = arguments.length, messages = Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
      messages[_key - 2] = arguments[_key];
    }

    return object[methodName].apply(object, messages);
  }
}

var DirectUploadController = function () {
  function DirectUploadController(input, file) {
    classCallCheck(this, DirectUploadController);

    this.input = input;
    this.file = file;
    this.directUpload = new DirectUpload(this.file, this.url, this);
    this.dispatch("initialize");
  }

  createClass(DirectUploadController, [{
    key: "start",
    value: function start(callback) {
      var _this = this;

      var hiddenInput = document.createElement("input");
      hiddenInput.type = "hidden";
      hiddenInput.name = this.input.name;
      this.input.insertAdjacentElement("beforebegin", hiddenInput);

      this.dispatch("start");

      this.directUpload.create(function (error, attributes) {
        if (error) {
          hiddenInput.parentNode.removeChild(hiddenInput);
          _this.dispatchError(error);
        } else {
          hiddenInput.value = attributes.signed_id;
        }

        _this.dispatch("end");
        callback(error);
      });
    }
  }, {
    key: "uploadRequestDidProgress",
    value: function uploadRequestDidProgress(event) {
      var progress = event.loaded / event.total * 100;
      if (progress) {
        this.dispatch("progress", { progress: progress });
      }
    }
  }, {
    key: "dispatch",
    value: function dispatch(name) {
      var detail = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

      detail.file = this.file;
      detail.id = this.directUpload.id;
      return dispatchEvent(this.input, "direct-upload:" + name, { detail: detail });
    }
  }, {
    key: "dispatchError",
    value: function dispatchError(error) {
      var event = this.dispatch("error", { error: error });
      if (!event.defaultPrevented) {
        alert(error);
      }
    }

    // DirectUpload delegate

  }, {
    key: "directUploadWillCreateBlobWithXHR",
    value: function directUploadWillCreateBlobWithXHR(xhr) {
      this.dispatch("before-blob-request", { xhr: xhr });
    }
  }, {
    key: "directUploadWillStoreFileWithXHR",
    value: function directUploadWillStoreFileWithXHR(xhr) {
      var _this2 = this;

      this.dispatch("before-storage-request", { xhr: xhr });
      xhr.upload.addEventListener("progress", function (event) {
        return _this2.uploadRequestDidProgress(event);
      });
    }
  }, {
    key: "url",
    get: function get$$1() {
      return this.input.getAttribute("data-direct-upload-url");
    }
  }]);
  return DirectUploadController;
}();

var inputSelector = "input[type=file][data-direct-upload-url]:not([disabled])";

var DirectUploadsController = function () {
  function DirectUploadsController(form) {
    classCallCheck(this, DirectUploadsController);

    this.form = form;
    this.inputs = findElements(form, inputSelector).filter(function (input) {
      return input.files.length;
    });
  }

  createClass(DirectUploadsController, [{
    key: "start",
    value: function start(callback) {
      var _this = this;

      var controllers = this.createDirectUploadControllers();

      var startNextController = function startNextController() {
        var controller = controllers.shift();
        if (controller) {
          controller.start(function (error) {
            if (error) {
              callback(error);
              _this.dispatch("end");
            } else {
              startNextController();
            }
          });
        } else {
          callback();
          _this.dispatch("end");
        }
      };

      this.dispatch("start");
      startNextController();
    }
  }, {
    key: "createDirectUploadControllers",
    value: function createDirectUploadControllers() {
      var controllers = [];
      this.inputs.forEach(function (input) {
        toArray$1(input.files).forEach(function (file) {
          var controller = new DirectUploadController(input, file);
          controllers.push(controller);
        });
      });
      return controllers;
    }
  }, {
    key: "dispatch",
    value: function dispatch(name) {
      var detail = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

      return dispatchEvent(this.form, "direct-uploads:" + name, { detail: detail });
    }
  }]);
  return DirectUploadsController;
}();

var processingAttribute = "data-direct-uploads-processing";
var submitButtonsByForm = new WeakMap();
var started = false;

function start() {
  if (!started) {
    started = true;
    document.addEventListener("click", didClick, true);
    document.addEventListener("submit", didSubmitForm);
    document.addEventListener("ajax:before", didSubmitRemoteElement);
  }
}

function didClick(event) {
  var target = event.target;

  if ((target.tagName == "INPUT" || target.tagName == "BUTTON") && target.type == "submit" && target.form) {
    submitButtonsByForm.set(target.form, target);
  }
}

function didSubmitForm(event) {
  handleFormSubmissionEvent(event);
}

function didSubmitRemoteElement(event) {
  if (event.target.tagName == "FORM") {
    handleFormSubmissionEvent(event);
  }
}

function handleFormSubmissionEvent(event) {
  var form = event.target;

  if (form.hasAttribute(processingAttribute)) {
    event.preventDefault();
    return;
  }

  var controller = new DirectUploadsController(form);
  var inputs = controller.inputs;


  if (inputs.length) {
    event.preventDefault();
    form.setAttribute(processingAttribute, "");
    inputs.forEach(disable);
    controller.start(function (error) {
      form.removeAttribute(processingAttribute);
      if (error) {
        inputs.forEach(enable);
      } else {
        submitForm(form);
      }
    });
  }
}

function submitForm(form) {
  var button = submitButtonsByForm.get(form) || findElement(form, "input[type=submit], button[type=submit]");

  if (button) {
    var _button = button,
        disabled = _button.disabled;

    button.disabled = false;
    button.focus();
    button.click();
    button.disabled = disabled;
  } else {
    button = document.createElement("input");
    button.type = "submit";
    button.style.display = "none";
    form.appendChild(button);
    button.click();
    form.removeChild(button);
  }
  submitButtonsByForm.delete(form);
}

function disable(input) {
  input.disabled = true;
}

function enable(input) {
  input.disabled = false;
}

function autostart() {
  if (window.ActiveStorage) {
    start();
  }
}

setTimeout(autostart, 1);

exports.start = start;
exports.DirectUpload = DirectUpload;

Object.defineProperty(exports, '__esModule', { value: true });

})));
